@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Sequence of numbers (for task_id > 0).
+## MCE::Core::Input::Generator - Sequence of numbers (for task_id > 0).
 ##
 ## This package provides a sequence of numbers used internally by the worker
 ## process. Distribution is divided equally among workers. This allows sequence
@@ -12,17 +12,19 @@
 
 package MCE::Core::Input::Generator;
 
-use strict;
-use warnings;
-
-our $VERSION = '1.867_01';
+our $VERSION = '1.500'; $VERSION = eval $VERSION;
 
 ## Items below are folded into MCE.
 
-package # hide from rpm
-   MCE;
+package MCE;
 
-no warnings qw( threads recursion uninitialized );
+use strict;
+use warnings;
+
+## Warnings are disabled to minimize bits of noise when user or OS signals
+## the script to exit. e.g. MCE_script.pl < infile | head
+
+no warnings 'threads'; no warnings 'uninitialized';
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -32,11 +34,13 @@ no warnings qw( threads recursion uninitialized );
 
 sub _worker_sequence_generator {
 
-   my ($self) = @_;
+   my MCE $self = $_[0];
 
    @_ = ();
 
-   _croak('MCE::_worker_sequence_generator: (user_func) is not specified')
+   die "Private method called" unless (caller)[0]->isa( ref($self) );
+
+   _croak("MCE::_worker_sequence_generator: 'user_func' is not specified")
       unless (defined $self->{user_func});
 
    my $_bounds_only = $self->{bounds_only} || 0;
@@ -64,8 +68,6 @@ sub _worker_sequence_generator {
 
    ## -------------------------------------------------------------------------
 
-   local $_;
-
    $self->{_last_jmp} = sub { goto _WORKER_SEQ_GEN__LAST; };
 
    if ($_begin == $_end) {                        ## Identical, yes.
@@ -73,9 +75,9 @@ sub _worker_sequence_generator {
       if ($_wid == 1) {
          $self->{_next_jmp} = sub { goto _WORKER_SEQ_GEN__LAST; };
 
-         $_ = (defined $_fmt) ? _sprintf("%$_fmt", $_next) : $_next;
+         local $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
 
-         if ($_chunk_size > 1 || $_bounds_only) {
+         if ($_chunk_size > 1) {
             $_ = ($_bounds_only) ? [ $_, $_ ] : [ $_ ];
          }
 
@@ -92,9 +94,7 @@ sub _worker_sequence_generator {
          return if ( $_flag && $_next > $_end);
          return if (!$_flag && $_next < $_end);
 
-         $_ = (defined $_fmt) ? _sprintf("%$_fmt", $_next) : $_next;
-         $_ = [ $_, $_ ] if ($_bounds_only);
-
+         local $_ = (defined $_fmt) ? sprintf("%$_fmt", $_next) : $_next;
          $_wuf->($self, $_, $_chunk_id);
 
          _WORKER_SEQ_GEN__NEXT_A:
@@ -108,88 +108,96 @@ sub _worker_sequence_generator {
       $self->{_next_jmp} = sub { goto _WORKER_SEQ_GEN__NEXT_B; };
 
       while (1) {
-         my @_n = (); my $_n_begin = $_next;
+         my $_n_begin = $_next; my @_n = ();
 
          ## -------------------------------------------------------------------
 
          if ($_bounds_only) {
-            my ($_tmp_b, $_tmp_e) = ($_next);
+            my $_tmp_b = $_next; my $_tmp_e;
 
-            if ($_begin <= $_end) {
-               if ($_step * ($_chunk_size - 1) + $_n_begin <= $_end) {
+            if ($_begin < $_end) {
+               if ($_step * $_chunk_size + $_n_begin <= $_end) {
                   $_tmp_e = $_step * ($_chunk_size - 1) + $_n_begin;
                }
-               elsif ($_step == 1) {
-                  $_tmp_e = $_end if ($_next <= $_end);
-               }
                else {
-                  for my $_i (1 .. $_chunk_size) {
+                  my $_start = int(
+                     ($_end - $_next) / $_chunk_size / $_step * $_chunk_size
+                  );
+                  $_start = 1 if ($_start < 1);
+
+                  for ($_start .. $_chunk_size) {
                      last if ($_next > $_end);
                      $_tmp_e = $_next;
-                     $_next  = $_step * $_i + $_n_begin;
+                     $_next  = $_step * $_ + $_n_begin;
                   }
                }
             }
             else {
-               if ($_step * ($_chunk_size - 1) + $_n_begin >= $_end) {
+               if ($_step * $_chunk_size + $_n_begin >= $_end) {
                   $_tmp_e = $_step * ($_chunk_size - 1) + $_n_begin;
                }
-               elsif ($_step == -1) {
-                  $_tmp_e = $_end if ($_next >= $_end);
-               }
                else {
-                  for my $_i (1 .. $_chunk_size) {
+                  my $_start = int(
+                     ($_next - $_end) / $_chunk_size / $_step * $_chunk_size
+                  );
+                  $_start = 1 if ($_start < 1);
+
+                  for ($_start .. $_chunk_size) {
                      last if ($_next < $_end);
                      $_tmp_e = $_next;
-                     $_next  = $_step * $_i + $_n_begin;
+                     $_next  = $_step * $_ + $_n_begin;
                   }
                }
             }
 
             return unless (defined $_tmp_e);
 
-            @_n = (defined $_fmt)
-               ? ( _sprintf("%$_fmt",$_tmp_b), _sprintf("%$_fmt",$_tmp_e) )
-               : ( $_tmp_b, $_tmp_e );
+            if (defined $_fmt) {
+               @_n = (sprintf("%$_fmt", $_tmp_b), sprintf("%$_fmt", $_tmp_e));
+            } else {
+               @_n = ($_tmp_b, $_tmp_e);
+            }
          }
 
          ## -------------------------------------------------------------------
 
          else {
-            if ($_begin <= $_end) {
-               if (!defined $_fmt && $_step == 1 && abs($_end) < ~1 && abs($_begin) < ~1) {
-                  @_n = ($_next + $_chunk_size <= $_end)
-                     ? ($_next .. $_next + $_chunk_size - 1)
-                     : ($_next .. $_end);
+            if ($_begin < $_end) {
+               if (!defined $_fmt && $_step == 1) {
+                  if ($_next + $_chunk_size <= $_end) {
+                     @_n = ($_next .. $_next + $_chunk_size - 1);
+                  } else {
+                     @_n = ($_next .. $_end);
+                  }
                }
                else {
-                  for my $_i (1 .. $_chunk_size) {
+                  for (1 .. $_chunk_size) {
                      last if ($_next > $_end);
 
                      push @_n, (defined $_fmt)
-                        ? _sprintf("%$_fmt", $_next) : $_next;
+                        ? sprintf("%$_fmt", $_next) : $_next;
 
-                     $_next = $_step * $_i + $_n_begin;
+                     $_next = $_step * $_ + $_n_begin;
                   }
                }
             }
             else {
-               for my $_i (1 .. $_chunk_size) {
+               for (1 .. $_chunk_size) {
                   last if ($_next < $_end);
 
                   push @_n, (defined $_fmt)
-                     ? _sprintf("%$_fmt", $_next) : $_next;
+                     ? sprintf("%$_fmt", $_next) : $_next;
 
-                  $_next = $_step * $_i + $_n_begin;
+                  $_next = $_step * $_ + $_n_begin;
                }
             }
 
-            return unless (scalar @_n);
+            return unless (@_n > 0);
          }
 
          ## -------------------------------------------------------------------
 
-         $_ = \@_n;
+         local $_ = \@_n;
          $_wuf->($self, \@_n, $_chunk_id);
 
          _WORKER_SEQ_GEN__NEXT_B:
@@ -206,37 +214,3 @@ sub _worker_sequence_generator {
 
 1;
 
-__END__
-
-###############################################################################
-## ----------------------------------------------------------------------------
-## Module usage.
-##
-###############################################################################
-
-=head1 NAME
-
-MCE::Core::Input::Generator - Sequence of numbers (for task_id > 0)
-
-=head1 VERSION
-
-This document describes MCE::Core::Input::Generator version 1.867_01
-
-=head1 DESCRIPTION
-
-This package provides a sequence of numbers used internally by the worker
-process. Distribution is divided equally among workers. This allows sequence
-to be configured independently among multiple user tasks.
-
-There is no public API.
-
-=head1 SEE ALSO
-
-The syntax for the C<sequence> option is described in L<MCE::Core>.
-
-=head1 AUTHOR
-
-Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
-
-=cut
-
