@@ -1,7 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Signal
-## -- Provides tmp_dir creation & signal handling for Many-Core Engine.
+## MCE::Signal - Temporary directory creation/cleanup & signal handling.
 ##
 ###############################################################################
 
@@ -10,9 +9,11 @@ package MCE::Signal;
 use strict;
 use warnings;
 
-use Fcntl qw( :flock );
+use Fcntl qw( :flock O_RDONLY );
 use base qw( Exporter );
 
+our $VERSION = '1.500'; $VERSION = eval $VERSION;
+
 our ($has_threads, $main_proc_id, $prog_name);
 our ($display_die_with_localtime, $display_warn_with_localtime);
 
@@ -24,8 +25,6 @@ BEGIN {
    $prog_name    =~ s{^.*[\\/]}{}g;
 }
 
-our $VERSION = '1.415'; $VERSION = eval $VERSION;
-
 our $tmp_dir = undef;
 
 our @EXPORT_OK = qw( $tmp_dir sys_cmd stop_and_exit );
@@ -437,11 +436,11 @@ __END__
 
 =head1 NAME
 
-MCE::Signal - Provides tmp_dir creation & signal handling for Many-Core Engine.
+MCE::Signal - Temporary directory creation/cleanup & signal handling
 
 =head1 VERSION
 
-This document describes MCE::Signal version 1.415
+This document describes MCE::Signal version 1.500
 
 =head1 SYNOPSIS
 
@@ -526,21 +525,15 @@ Nothing is exported by default. Exportable are 1 variable and 2 subroutines.
  use MCE::Signal qw( -keep_tmp_dir );
  use MCE::Signal qw( -use_dev_shm -keep_tmp_dir );
 
-=head1 REQUIREMENTS
+=head1 INDEX
 
-Perl 5.8.0 or later
-
-=head1 SOURCE
-
-The source is hosted at: L<http://code.google.com/p/many-core-engine-perl/>
+L<MCE>
 
 =head1 AUTHOR
 
 Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
-=head1 COPYRIGHT AND LICENSE
-
-Copyright (C) 2012-2013 by Mario E. Roy
+=head1 LICENSE
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
@@ -549,3 +542,4 @@ by the Free Software Foundation; or the Artistic License.
 See L<http://dev.perl.org/licenses/> for more information.
 
 =cut
+
